@@ -1,6 +1,6 @@
 // -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
 //
-// Copyright 2011-2014, Julian Catchen <jcatchen@uoregon.edu>
+// Copyright 2011-2018, Julian Catchen <jcatchen@illinois.edu>
 //
 // This file is part of Stacks.
 //
@@ -105,7 +105,7 @@ int parse_illumina_v2(const char *file) {
 }
 
 int
-parse_input_record(Seq *s, Read *r)
+parse_input_record(Seq *s, RawRead *r)
 {
     char *p, *q, *z;
     uint  lim;
@@ -145,7 +145,11 @@ parse_input_record(Seq *s, Read *r)
 
         // Run number.
         for (p = q+1, q = p; *q != ':' && q < stop; q++);
-        //*q = '\0';
+        if (q < stop) {
+            *q = '\0';
+            r->run = atoi(p);
+            *q = ':';
+        }
 
         // Flowcell ID.
         for (p = q+1, q = p; *q != ':' && q < stop; q++);
@@ -160,9 +164,9 @@ parse_input_record(Seq *s, Read *r)
 
         for (p = q+1, q = p; *q != ':' && q < stop; q++);
         if (q < stop) {
-        *q = '\0';
-        r->tile = atoi(p);
-        *q = ':';
+            *q = '\0';
+            r->tile = atoi(p);
+            *q = ':';
         }
 
         for (p = q+1, q = p; *q != ':' && q < stop; q++);
@@ -233,19 +237,30 @@ parse_input_record(Seq *s, Read *r)
                     strncpy(r->index_bc, p, lim);
                     r->index_bc[lim] = '\0';
                     break;
+                case null_index:
+		    if (*z == '+')
+			p = z + 1;
+		    for (z = p; *z != '\0'; z++);
+		    lim = z - p;
+		    lim = lim < max_bc_size_1 ? lim : max_bc_size_1;
+		    strncpy(r->index_bc, p,  lim);
+                    r->index_bc[lim] = '\0';
+                    break;		    
                 default:
                     break;
                 }
             } else if (r->read == 2) {
                 if (*z == '+')
                     p = z + 1;
+		for (z = p; *z != '\0'; z++);
+		lim = z - p;
 
                 switch (barcode_type) {
-                case null_index:
                 case index_index:
                 case inline_index:
-                    strncpy(r->index_bc, p,  max_bc_size_2);
-                    r->index_bc[max_bc_size_2] = '\0';
+		    lim = lim < max_bc_size_2 ? lim : max_bc_size_2;
+                    strncpy(r->index_bc, p,  lim);
+                    r->index_bc[lim] = '\0';
                     break;
                 default:
                     break;
@@ -323,25 +338,31 @@ parse_input_record(Seq *s, Read *r)
     r->len = len;
 
     if (r->read == 1) {
-            switch (barcode_type) {
-            case inline_null:
-            case inline_inline:
-            case inline_index:
-                strncpy(r->inline_bc, r->seq, max_bc_size_1);
-                r->inline_bc[max_bc_size_1] = '\0';
-                break;
-            case index_inline:
-                strncpy(r->inline_bc, r->seq, max_bc_size_2);
-                r->inline_bc[max_bc_size_2] = '\0';
-                break;
-            default:
-                break;
-            }
-    } else if (r->read == 2 &&
-               (barcode_type == inline_inline ||
-                barcode_type == index_inline)) {
-        strncpy(r->inline_bc, r->seq, max_bc_size_2);
-        r->inline_bc[max_bc_size_2] = '\0';
+        switch (barcode_type) {
+        case inline_null:
+        case inline_inline:
+        case inline_index:
+            strncpy(r->inline_bc, r->seq, max_bc_size_1);
+            r->inline_bc[max_bc_size_1] = '\0';
+            break;
+        case index_inline:
+            strncpy(r->inline_bc, r->seq, max_bc_size_2);
+            r->inline_bc[max_bc_size_2] = '\0';
+            break;
+        default:
+            break;
+        }
+    } else if (r->read == 2) {
+        switch (barcode_type) {
+        case null_inline:
+        case inline_inline:
+        case index_inline:
+            strncpy(r->inline_bc, r->seq, max_bc_size_2);
+            r->inline_bc[max_bc_size_2] = '\0';
+            break;
+        default:
+            break;
+        }
     }
 
     r->retain = 1;
@@ -422,7 +443,7 @@ reverse_qual(char *qual, int offset, bool overhang)
 // Functions for quality filtering based on phred scores.
 //
 int
-check_quality_scores(Read *href, int qual_offset, int score_limit, int len_limit, int offset)
+check_quality_scores(RawRead *href, int qual_offset, int score_limit, int len_limit, int offset)
 {
     //
     // Phred quality scores are discussed here:
@@ -512,7 +533,7 @@ check_quality_scores(Read *href, int qual_offset, int score_limit, int len_limit
 }
 
 bool
-correct_barcode(set<string> &bcs, Read *href, seqt type, int num_errs)
+correct_barcode(set<string> &bcs, RawRead *href, seqt type, int num_errs)
 {
     if (recover == false)
         return false;
@@ -596,7 +617,7 @@ init_adapter_seq(int kmer_size, char *adapter, int &adp_len, AdapterHash &kmers)
 }
 
 int
-filter_adapter_seq(Read *href, char *adapter, int adp_len, AdapterHash &adp_kmers,
+filter_adapter_seq(RawRead *href, char *adapter, int adp_len, AdapterHash &adp_kmers,
                    int kmer_size, int distance, int len_limit)
 {
     vector<pair<int, int> > hits;
