@@ -1,6 +1,6 @@
 // -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
 //
-// Copyright 2012-2014, Julian Catchen <jcatchen@uoregon.edu>
+// Copyright 2012-2021, Julian Catchen <jcatchen@illinois.edu>
 //
 // This file is part of Stacks.
 //
@@ -626,6 +626,9 @@ load_barcodes(string barcode_file, vector<BarcodePair> &barcodes,
     case inline_null:
         cerr << "Searching for single-end, inlined barcodes.\n";
         break;
+    case null_inline:
+        cerr << "Searching for paired-end, inlined barcodes.\n";
+        break;
     case index_index:
         cerr << "Searching for single and paired-end, indexed barcodes.\n";
         break;
@@ -851,7 +854,8 @@ load_barcodes(string barcode_file, vector<BarcodePair> &barcodes,
         }
     } else {
         if (barcode_type != inline_null &&
-            barcode_type != index_null) {
+            barcode_type != index_null  &&
+	    barcode_type != null_index) {
             cerr << "You provided single-end barcodes but did not specify a single-end barcode type.\n";
             help();
         }
